from panda3d.core import *
from direct.showbase.DirectObject import DirectObject
from direct.interval.IntervalGlobal import *

class CameraControler (DirectObject):
    def __init__(self, root, walls, pos=(0,0,1), offset=(0, -8, 9), speed=50.0, angle=-45, zoomspeed=0.05):    
        self.root=root
        self.walls=walls
        self.skip=False
        self.cameraNode  = render.attachNewNode("cameraNode")
        self.cameraNode.setPos(pos)
        self.cameraGimbal  = self.cameraNode.attachNewNode("cameraGimbal")
        base.camera.setPos(pos)
        base.camera.setX(base.camera, offset[0])
        base.camera.setY(base.camera, offset[1])
        base.camera.setZ(offset[2])
        base.camera.setP(angle)
        base.camera.wrtReparentTo(self.cameraGimbal)        
        
        
        self.keyMap = {'rotate': False}  
        self.accept('mouse1', self.keyMap.__setitem__, ['rotate', True])                
        self.accept('mouse1-up', self.keyMap.__setitem__, ['rotate', False])
               
        self.mouseSpeed=speed #default 50.0
        self.accept('wheel_up', self.zoom_control,[zoomspeed])
        self.accept('wheel_down',self.zoom_control,[-zoomspeed])
        
        #self.cameraNode.wrtReparentTo(root)        
        
        #the collision ray needs to be movend up a bit, so it will not 
        #collide with the the floor, just the walls     
        ray=CollisionSegment(pos[0],pos[1],pos[2]+1, offset[0],offset[1],offset[2])
        self.cam_ray=render.attachNewNode(CollisionNode('camRay'))       
        self.cam_ray.node().addSolid(ray)   
        self.cam_ray.show()
        self.cam_ray.wrtReparentTo(self.cameraGimbal)
        self.cam_ray.setPythonTag('camRay', True)
        self.cam_ray.node().setIntoCollideMask(BitMask32.allOff()) 
        self.traverser=CollisionTraverser("camTrav")   
        #self.traverser.showCollisions(render)
        self.queue = CollisionHandlerQueue()
        self.traverser.addCollider(self.cam_ray, self.queue)
        
        
        self.cameraNode.setH(-45)
        
        taskMgr.add(self.update, "camcon_update", sort=-1)
        taskMgr.doMethodLater(0.2, self.turn, 'turnTask')
    
    def turn(self, task):
        pos=self.root.getPos(render)
        LerpPosInterval(self.cameraNode, 0.2, pos).start()
        return task.again 
        
    def zoom(self, t):
        base.camera.setY(base.camera, t)
    
    def zoom_control(self, amount):
        LerpFunc(self.zoom,fromData=0,toData=amount, duration=.5, blendType='easeOut').start()
        
    def update(self, task): 
        self.cameraNode.setHpr(self.root.getHpr(render))
        if base.mouseWatcherNode.hasMouse():
            deltaX = base.mouseWatcherNode.getMouseX()  # mouse distance from the center
            deltaY = base.mouseWatcherNode.getMouseY()
            if self.keyMap['rotate']:        
                base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2)
                if not self.skip:
                    self.cameraGimbal.setH(self.cameraGimbal.getH()- deltaX*self.mouseSpeed)
                    self.cameraGimbal.setP(self.cameraGimbal.getP()+ deltaY*self.mouseSpeed)
                self.skip=False
            else:
                self.skip=True         
        self.traverser.traverse(render)
        for wall in self.walls:
            wall.show()
        for entry in self.queue.getEntries():
            if entry.getFromNodePath().hasPythonTag('camRay'):
                if entry.getIntoNodePath().hasPythonTag('wall'):
                    entry.getIntoNodePath().getPythonTag('wall').hide()           
         
        return task.cont 