from panda3d.core import loadPrcFileData
loadPrcFileData('','show-frame-rate-meter  #f') 
from direct.showbase.AppRunnerGlobal import appRunner
if appRunner: #run from binary
    path=appRunner.p3dFilename.getDirname()+'/'
else:  #run from python 
    path=''
try:    #should rewrite using 'with open()'
    f = open(path+'config.txt', 'r')
    for line in f:        
        if not line.startswith('#'):        
            loadPrcFileData('',line)
except IOError:
    print "No config file, using default" 

from panda3d.core import WindowProperties
from panda3d.core import ConfigVariableInt
from panda3d.core import ConfigVariableBool
from panda3d.core import ConfigVariableString 
#add new options here
config_options=[
                ('bool','fullscreen', 0),
                ('int','win-size', '640 480'),
                ('string','post-process', 'Bloom+Lens Flare'),
                ('string','antialiasing', 'FXAA'),                
                ('int','reflection', 512),
                ('int','music-volume', 50),
                ('int','sound-volume', 75),
                ('int','max-colors', 3),
                ('int','cube-per-face', 16),
                ('string','music_track', 'music.ogg'),
                ('string','key_forward', 'w'),                
                ('string','key_left', 'a'),
                ('string','key_right', 'd')         
                ]
config={}                
for option in config_options:
    if option[0]=='int':        
        var=ConfigVariableInt(option[1], option[2])        
        config[option[1]]=var.getValue()
        #special case for 'win-size'
        if option[1]=='win-size':
            config[option[1]]=(var.getWord(0), var.getWord(1))
    elif option[0]=='bool':
        var=ConfigVariableBool(option[1], option[2])
        config[option[1]]=var.getValue()
    elif option[0]=='string':
        var=ConfigVariableString(option[1], option[2])
        config[option[1]]=var.getValue() 
#set multisample here, or none at all... a bit buggy        
if config['antialiasing']=="MSAA x2":
    loadPrcFileData('','multisamples 2') 
elif config['antialiasing']=="MSAA x4":
    loadPrcFileData('','multisamples 4') 
elif config['antialiasing']=="MSAA x8":
    loadPrcFileData('','multisamples 8')     
elif config['antialiasing']=="MSAA x16":
    loadPrcFileData('','multisamples 16') 
elif config['antialiasing']=="FXAA":    
    loadPrcFileData('','textures-power-2 None')
else:    
    loadPrcFileData('','multisamples 0')
    
wp = WindowProperties.getDefault() 
wp.setOrigin(-2,-2)  
wp.setTitle("Room Wezu@PyWeek19") 
#wp.setUndecorated(True)        
if(config['fullscreen']): 
    wp.setFullscreen(True)
wp.setSize(config['win-size'][0],config['win-size'][1])    
WindowProperties.setDefault(wp)
    
from panda3d.core import *
from direct.filter.FilterManager import FilterManager    
from direct.showbase.DirectObject import DirectObject
import direct.directbase.DirectStart
from direct.interval.IntervalGlobal import *
from gfx import setup_gfx
from cameracontroler import CameraControler
from player import PC
import random

class World(DirectObject):
    def __init__(self):  
        render.setShaderAuto()
        base.cam.node().getLens().setNearFar(1.0, 60.0)
        base.setBackgroundColor(0, 0, 0)
        
        #music
        base.musicManager.setVolume(config['music-volume']*0.01)
        self.music=base.loadMusic("music/"+config['music_track'])
        self.music.setLoop(True)
        self.music.play()
        base.sfxManagerList[0].setVolume(config['sound-volume']*0.01)
        
        #load the room
        room=loader.loadModel("models/room2")
        room.reparentTo(render)
        #self.room.setScale(10)
        room.setShader(Shader.load("shaders/default.cg"))
        room.setShaderInput("reflect",1.0)
        
                       
        #light setup
        self.pLight = PointLight('plight')                
        self.pLight.setColor(VBase4(0.8, 0.8, 1, 1))        
        self.pLight.setAttenuation(Point3(0.0, 0.5, 0.0))         
        self.pLightNode = render.attachNewNode(self.pLight) 
        self.pLightNode.setPos(3.5,3.5, 4.5)
        render.setLight(self.pLightNode)        
        self.sLight=Spotlight('sLight')        
        self.sLight.setColor(VBase4(0.3, 0.3, 0.3, 1))        
        spot_lens = PerspectiveLens()
        spot_lens.setFov(40)
        self.sLight.setLens(spot_lens)
        #self.sLight.showFrustum() 
        self.Ambient = render.attachNewNode( self.sLight)        
        self.Ambient.setPos(0, 0, 0)
        self.Ambient.wrtReparentTo(base.camera)
        render.setLight(self.Ambient)
        render.setShaderInput("slight0", self.Ambient)        
        render.setShaderInput("plight0", self.pLightNode)        
        
        
        #box proto, it's faster to copy then to load a new one (and compile shaders)
        self.box=loader.loadModel("models/box1")
        self.box.setShader(Shader.load("shaders/default.cg"))
        self.box.setShaderInput("reflect",1.0)
        self.box.setTransparency(TransparencyAttrib.MDual)
        #valid places for the boxes
        grid=[(1,1,0),(1,2,0),(1,3,0),(1,4,0),(1,5,0),(1,6,0),
              (2,1,0),(2,2,0),(2,3,0),(2,4,0),(2,5,0),(2,6,0),
              (3,1,0),(3,2,0),(3,3,0),(3,4,0),(3,5,0),(3,6,0),
              (4,1,0),(4,2,0),(4,3,0),(4,4,0),(4,5,0),(4,6,0),
              (5,1,0),(5,2,0),(5,3,0),(5,4,0),(5,5,0),(5,6,0),
              (6,1,0),(6,2,0),(6,3,0),(6,4,0),(6,5,0),(6,6,0)
             ]
                
        #walls and stuff
        self.center=render.attachNewNode("center")
        self.center.setPos(3.5,3.5, 4.5)
       
        self.walls=[]
        self.boxes=[]
        print 'Generating level',
        for i in range(6):
            print '.',
            self.walls.append(render.attachNewNode("wall_center"+str(i)))    
            self.walls[-1].setPos(3.5,3.5, 4.5)
            wall=render.attachNewNode("wall"+str(i))
            wall.wrtReparentTo(self.walls[-1])
            coll=render.attachNewNode(CollisionNode('wall_collision'+str(i)))            
            quad = CollisionPolygon(Point3(-2, 9, 0),Point3(-2, -2, 0),Point3(9, -2, 0),Point3(9, 9, 0))
            coll.node().addSolid(quad) 
            #coll.show()
            coll.wrtReparentTo(self.walls[-1])
            coll.setPythonTag('wall', self.walls[-1]) 
            coll.setPythonTag('wallID', i)            
            for j in random.sample(grid, config['cube-per-face']):
                self.createBox(wall, j, i)                
        self.walls[1].setR(90)       
        self.walls[2].setR(180)
        self.walls[3].setR(270)
        self.walls[4].setHpr(90, 270, 90)
        self.walls[5].setHpr(-90, 90, 90)        
        print 'done!'       
        
        #player control
        self.pc=PC(keymap=config, boxes=self.boxes)
        
        #camera control
        base.disableMouse()  
        self.cc=CameraControler(root=self.pc.cameraNode, walls=self.walls, pos=(0,0,0), offset=(0,-8,9), speed=50.0, angle=-45, zoomspeed=0.05)        
                       
        self.accept( 'window-event', self.windowEventHandler) 
        self.winsize=None
        self.Manager=FilterManager(base.win, base.cam)    

    def createBox(self, parent, pos, wall):
        print '.',
        self.boxes.append(self.box.copyTo(parent))
        self.boxes[-1].setPos(pos)         
        color=random.randint(0, config['max-colors'])            
        self.boxes[-1].setTexture(self.boxes[-1].findTextureStage('color'), loader.loadTexture("models/box"+str(color)+"_cg.png"), 1 )
        coll_box=self.boxes[-1].attachNewNode(CollisionNode('box')) 
        coll_box.node().addSolid(CollisionSphere(0, 0, 1, 0.5))  
        coll_box.node().setPythonTag('color', color)
        coll_box.node().setPythonTag('box', self.boxes[-1])        
        self.boxes[-1].setPythonTag('color', color)
        
        
    def windowEventHandler( self, window=None ):    
        if window is not None: # window is none if panda3d is not started
            wp=window.getProperties()       
            newsize=[wp.getXSize(),wp.getYSize()]
            if self.winsize!=newsize:        
                #resizing the window breaks the filter manager, so I just make a new one
                self.Manager.cleanup()
                self.Manager=FilterManager(base.win, base.cam)    
                setup_gfx(config, self.Manager)
                self.winsize=newsize    
w = World()
run()    