from panda3d.core import *
from direct.showbase.DirectObject import DirectObject
from direct.actor.Actor import Actor
from direct.interval.IntervalGlobal import *
from direct.showbase.PythonUtil import fitSrcAngle2Dest
from score import HiScore
from vfx import vfx
import random 

class PC(DirectObject):    
    
    def __init__(self,keymap, boxes, pos=(0,0,0), actor_scale=0.02):
        self.Score=HiScore()
        self.boxes=boxes
        self.myWall=0
        self.playerNode=render.attachNewNode("playerNode")    
        self.playerNode.setH(180)
        self.playerNode.setPos(render, pos)
        #actor
        self.actor=Actor('models/pc',
                        {'walk':'models/pc_walk1',
                        'turn1':'models/pc_turn1',
                        'turn2':'models/pc_turn2',
                        'climb':'models/pc_climb' }) 
        self.actor.setBlend(frameBlend = True) 
        self.actor.setPlayRate(2.0, "walk")   
        self.cameraNode=render.attachNewNode("cameraNode") 
        #self.cameraNode.setHpr(180,0,0)        
        self.cameraNode.wrtReparentTo(self.actor.exposeJoint(None, 'modelRoot', 'camera_bone'))  
        self.actor.setShader(Shader.load("shaders/default.cg"))
        self.actor.setShaderInput("reflect",1.0)                
        self.actor.reparentTo(self.playerNode) 
        self.actor.setScale(actor_scale)        
        self.playerNode.wrtReparentTo(render)        
        
        #key mapping:
        self.accept(keymap['key_left'],self.turn, [90])
        self.accept(keymap['key_right'],self.turn, [-90])
        self.accept(keymap['key_forward'],self.move, [-1])
        #self.accept(keymap['key_back'],self.move, [1])
        #self.accept(keymap['key_use'],self.use)        
              
        #Game Variables:
        self.isIdle=True
        self.nextColor=-1
        self.currentColor=-1
        self.lastBox=None  
        self.selectedBoxes=[]  
        self.multiwalls=[]
        #collision
        self.queue = CollisionHandlerQueue()
        self.traverser=CollisionTraverser("pcTrav")  
      
        ray2=CollisionSegment(0,0,1, 0,1.1,1)        
        self.pc_ray=render.attachNewNode(CollisionNode('pcRay'))       
        self.pc_ray.node().addSolid(ray2)   
        #self.pc_ray.show()
        self.pc_ray.wrtReparentTo(self.playerNode)
        self.pc_ray.node().setIntoCollideMask(BitMask32.allOff()) 
        self.pc_ray.setPythonTag('pcRay1', True)
        self.traverser.addCollider(self.pc_ray, self.queue)
        
        ray3=CollisionSegment(0,0,1, 0,0,-1)        
        self.pc_ray2=render.attachNewNode(CollisionNode('pcRay2'))       
        self.pc_ray2.node().addSolid(ray3)   
        #self.pc_ray2.show()
        self.pc_ray2.wrtReparentTo(self.playerNode)
        self.pc_ray2.node().setIntoCollideMask(BitMask32.allOff()) 
        self.pc_ray2.setPythonTag('pcRay2', True)
        self.traverser.addCollider(self.pc_ray2, self.queue)   
        
        #sound &voice
        self.useSFX=base.loader.loadSfx("sfx/sfx0.ogg")
        self.taunt=[base.loader.loadSfx("voice/bad0.ogg"),
                    base.loader.loadSfx("voice/bad1.ogg"),
                    base.loader.loadSfx("voice/bad2.ogg"),
                    base.loader.loadSfx("voice/bad3.ogg"),
                    base.loader.loadSfx("voice/bad4.ogg"),
                    base.loader.loadSfx("voice/bad5.ogg"),
                    base.loader.loadSfx("voice/bad6.ogg"),
                    base.loader.loadSfx("voice/bad7.ogg"),
                    base.loader.loadSfx("voice/bad8.ogg"),
                    base.loader.loadSfx("voice/bad9.ogg"),
                    base.loader.loadSfx("voice/bad10.ogg"),
                    base.loader.loadSfx("voice/bad11.ogg"),
                    base.loader.loadSfx("voice/bad12.ogg")
                    ]
        self.praise=[base.loader.loadSfx("voice/good0.ogg"),
                    base.loader.loadSfx("voice/good1.ogg"),
                    base.loader.loadSfx("voice/good2.ogg"),
                    base.loader.loadSfx("voice/good3.ogg"),
                    base.loader.loadSfx("voice/good4.ogg"),
                    base.loader.loadSfx("voice/good5.ogg"),
                    base.loader.loadSfx("voice/good6.ogg"),
                    base.loader.loadSfx("voice/good7.ogg"),
                    base.loader.loadSfx("voice/good8.ogg"),
                    base.loader.loadSfx("voice/good9.ogg"),
                    base.loader.loadSfx("voice/good10.ogg"),
                    base.loader.loadSfx("voice/good11.ogg"),
                    base.loader.loadSfx("voice/good12.ogg")
                    ]
                    
    def vanishBox(self, box):
        box.detachNode()
        
    def levelCamera(self):
        self.cameraNode.setH(90*round(self.cameraNode.getH()/90))
        self.cameraNode.setP(90*round(self.cameraNode.getP()/90))
        self.cameraNode.setR(90*round(self.cameraNode.getR()/90))
    
    def setIdle(self, mode):
        self.isIdle=mode 
        
    def runTraverser(self):
        self.traverser.traverse(render)
        for entry in self.queue.getEntries():                        
            if entry.getFromNodePath().hasPythonTag('pcRay2'):        
                self.girdPos= entry.getSurfacePoint(entry.getIntoNodePath())
                self.myWall= entry.getIntoNodePath().getPythonTag('wallID')
            if entry.getFromNodePath().hasPythonTag('pcRay1'):
                if entry.getIntoNodePath().hasPythonTag('color'):
                    self.nextColor= entry.getIntoNodePath().getPythonTag('color')                     
                    self.lastBox=entry.getIntoNodePath().getPythonTag('box')
                    
    def checkColor(self):
        self.traverser.traverse(render)
        for entry in self.queue.getEntries():            
            if entry.getFromNodePath().hasPythonTag('pcRay1'):
                if entry.getIntoNodePath().hasPythonTag('color'):
                    self.nextColor=entry.getIntoNodePath().getPythonTag('color')
                    self.lastBox=entry.getIntoNodePath().getPythonTag('box')
                    
    def turn(self, direction):
        if not self.isIdle:
            return
        self.nextColor=-1
        self.playerNode.setH(self.playerNode, direction)
        if direction<0:
            self.actor.play('turn1')
        else:    
            self.actor.play('turn2')
        Sequence(Func(self.setIdle, False), Wait(0.7), Func(self.setIdle, True), Func(self.levelCamera)).start()
        self.checkColor()   
    
    def move(self, direction): 
        if not self.isIdle:
            return
        self.traverser.traverse(render)
        for entry in self.queue.getEntries():            
            if entry.getFromNodePath().hasPythonTag('pcRay1'):  
                if entry.getIntoNodePath().hasPythonTag('wallID'):
                    self.girdPos= entry.getSurfacePoint(entry.getIntoNodePath())
                    self.myWall= entry.getIntoNodePath().getPythonTag('wallID')
                    self.playerNode.setP(self.playerNode,-90)
                    self.playerNode.setY(self.playerNode, direction)
                    self.playerNode.setZ(self.playerNode, -1)
                    self.actor.play('climb')
                    self.checkColor()
                    Sequence(Func(self.setIdle, False), Wait(0.45), Func(self.setIdle, True)).start()
                    return
            if entry.getFromNodePath().hasPythonTag('pcRay2'):        
                self.girdPos= entry.getSurfacePoint(entry.getIntoNodePath())
                self.myWall= entry.getIntoNodePath().getPythonTag('wallID')
                
        if self.nextColor==-1:
            if not self.isIdle:
                return
            self.actor.play('walk')
            #self.playerNode.setY(self.playerNode, direction)   
            #this is to counter the 'phantom move'    
            Sequence(Func(self.setIdle, False),
                     Wait(0.01),
                     Func(self.playerNode.setY,self.playerNode, direction),
                     Wait(0.45),
                     Func(self.runTraverser),
                     Func(self.setIdle, True),
                     ).start()                
        else:
            if self.currentColor==-1:  
                self.useSFX.play()
                self.selectedBoxes.append(self.lastBox)
                self.multiwalls.append(self.myWall)
                for box in self.boxes:
                    if box!=self.lastBox:
                        box.setTexture(box.findTextureStage('color'), loader.loadTexture("models/box_black_cg.png"), 1)
                        box.setTexture(box.findTextureStage('normal'), loader.loadTexture("models/box_black_ns.png"), 1)
                print self.nextColor
                self.currentColor=self.nextColor
            elif self.currentColor==self.nextColor:
                if self.lastBox in self.selectedBoxes:
                    return
                if not self.myWall in self.multiwalls:
                    self.multiwalls.append(self.myWall)                    
                self.Score.Multiplier(len(self.multiwalls))        
                self.selectedBoxes.append(self.lastBox)
                self.Score.Set(len(self.selectedBoxes))
                self.lastBox.setTexture(self.lastBox.findTextureStage('color'), loader.loadTexture("models/box"+str(self.currentColor)+"_cg.png"), 1)
                self.lastBox.setTexture(self.lastBox.findTextureStage('normal'), loader.loadTexture("models/box_ns.png"), 1)
                random.choice(self.praise).play()
                self.useSFX.play()
            else:
                self.currentColor=-1
                for box in self.boxes:
                    color=str(box.getPythonTag('color'))
                    box.setTexture(box.findTextureStage('color'), loader.loadTexture("models/box"+color+"_cg.png"), 1)
                    box.setTexture(box.findTextureStage('normal'), loader.loadTexture("models/box_black_ns.png"), 1)
                for box in self.selectedBoxes:
                    Sequence(
                            Func(self.setIdle, False),
                            LerpColorInterval(box, 0.5, (1,1,1,0), (1,1,1,1)),
                            Func(box.detachNode),
                            Func(self.setIdle, True),
                            ).start()
                    #box.detachNode()
                self.Score.Bank()    
                #print len(self.selectedBoxes)*len(self.multiwalls)    
                self.selectedBoxes=[]    
                self.multiwalls=[]
                random.choice(self.taunt).play()
                self.useSFX.play()
            return        

      